import { v4 as uuidv4 } from 'uuid';
import {  Queue} from 'data-struct-js'
/**
 * ws类
 */
class ReactWebsocket  {
  constructor() {
    //首次使用构造器实例
    if (!ReactWebsocket.instance) {
      this.ws=new WebSocket("ws://127.0.0.1:1323/ws");
      this.callbackMap= new Map();
      this.noMessageSent= new Queue();
      this.ws.onmessage=((evt)=>{this.OnMessage(evt)})
      this.ws.onopen=((evt)=>{this.OnOpen(evt)})
      this.ws.onclose = function(event) {
        console.log("WebSocket is closed now.");
      };
      //将this挂载到SingletonApple这个类的instance属性上
      ReactWebsocket.instance = this;
    }
    return ReactWebsocket.instance;
  }
/**
 * 发送ws
 * @param {string} method 方法 string
 * @param {*} data   数据 obj
 * @param {function} callback 回调函数 function
 */
  sendMessage(method,data,callback) {
    let uuid=uuidv4()
    let res = { method, data, uuid }
    let result = JSON.stringify(res);
    // console.log(result)
    this.callbackMap.set(uuid,callback)
    if(this.ws.readyState==1){
      this.ws.send(result);
    }else{
      this.noMessageSent.enqueue(result)
    }

  }
  /**
   * 关闭ws
   * @param {*} code 
   * @param {*} reason 
   */
  closeSocket(code, reason) {
    this.ws.close(code, reason);
  }
  /**
   * 监听ws消息
   * @param {*} res 
   */
  OnMessage(res){
    let obj=JSON.parse(res.data)
    if(this.callbackMap.has(obj.uuid)){
      let callback=this.callbackMap.get(obj.uuid)
      callback(obj)
      this.callbackMap.delete(obj.uuid)
    }else{
      //广播
    }

  }
  OnOpen(evt){
    while(1){
      let result =  this.noMessageSent.dequeue()
      if(result){
        this.ws.send(result)
      }else{
        break
      }
    }
  }

}
export default ReactWebsocket;